/*
idmefxml_parse.h: header file for libidmef parsing functions
Author: Adam Migus, NAI Labs, (amigus@nai.com)

Copyright (c) 2001 Networks Associates Technology, Inc.
All rights reserved

This library is released under the GNU GPL and BSD software licenses.
You may choose to use one or the other, BUT NOT BOTH.  The GNU GPL
license is located in the file named COPYING.  The BSD license is located
in the file named COPYING.BSD.  Please contact us if there are any
questions.

*/

#ifndef _IDMEFXML_PARSE_H
#define _IDMEFXML_PARSE_H

/* $Id: idmefxml_parse.h,v 1.10 2002/02/20 23:54:28 amigus Exp $ */

#include "idmefxml_types.h"

IDMEFmessage * get_idmef_message_from_file(char *filename);
IDMEFmessage * get_idmef_message(char *buffer, int size);

char * node_category_as_string(IDMEFnode_category category);
IDMEFnode_category get_node_category(char *p);

char * address_category_as_string(IDMEFaddress_category category);
IDMEFaddress_category get_address_category(char *p);

IDMEFspoofed get_spoofed(char *p);
char * spoofed_as_string(IDMEFspoofed spoofed);

IDMEFdecoy get_decoy(char *p);
char * decoy_as_string(IDMEFdecoy decoy);

IDMEFuser_category get_user_category(char *p);
char * user_category_as_string(IDMEFuser_category category);

IDMEFUserId_type get_userid_type(char *p);
char * userid_type_as_string(IDMEFUserId_type type);

IDMEForigin get_classification_origin(char *p);
char * classification_origin_as_string(IDMEForigin origin);

IDMEFadditionaldata_type get_additionaldata_type(char *p);
char * additionaldata_type_as_string(IDMEFadditionaldata_type type);

IDMEFimpact_severity get_impact_severity(char *p);
char * impact_severity_as_string(IDMEFimpact_severity severity);

IDMEFimpact_completion get_impact_completion(char *p);
char * impact_completion_as_string(IDMEFimpact_completion completion);

IDMEFimpact_type get_impact_type(char *p);
char * impact_type_as_string(IDMEFimpact_type type);

IDMEFaction_category get_action_category(char *p);
char * action_category_as_string(IDMEFaction_category category);

IDMEFconfidence_rating get_confidence_rating(char *p);
char * confidence_rating_as_string(IDMEFconfidence_rating rating);

IDMEFfile_category get_file_category(char *p);
char * file_category_as_string(IDMEFfile_category category);

IDMEFlinkage_category get_linkage_category(char *p);
char * linkage_category_as_string(IDMEFlinkage_category category);

int parse_userid(xmlNodePtr cur, IDMEFuserid *userid);
int parse_user(xmlNodePtr cur, IDMEFuser *user);
int parse_time(xmlNodePtr cur, IDMEFtime *time, char *name);
int parse_address(xmlNodePtr cur, IDMEFaddress *address);
int parse_process(xmlNodePtr cur, IDMEFprocess *process);
int parse_additionaldata(xmlNodePtr cur, IDMEFadditionaldata *additionaldata);
int parse_classification(xmlNodePtr cur, IDMEFclassification *classification);
int parse_impact(xmlNodePtr cur, IDMEFimpact *impact);
int parse_action(xmlNodePtr cur, IDMEFaction *action);
int parse_confidence(xmlNodePtr cur, IDMEFconfidence *confidence);
int parse_assessment(xmlNodePtr cur, IDMEFassessment *assessment);
int parse_node(xmlNodePtr cur, IDMEFnode *node);
int parse_webservice(xmlNodePtr cur, IDMEFwebservice *webservice);
int parse_snmpservice(xmlNodePtr cur, IDMEFSNMPservice *snmpservice);
int parse_service(xmlNodePtr cur, IDMEFservice *service);
int parse_linkage(xmlNodePtr cur, IDMEFlinkage *linkage);
int parse_inode(xmlNodePtr cur, IDMEFinode *inode);
int parse_fileaccess(xmlNodePtr cur, IDMEFfileaccess *fileaccess);
int parse_file(xmlNodePtr cur, IDMEFfile *file);
int parse_filelist(xmlNodePtr cur, IDMEFfilelist *filelist);
int parse_target(xmlNodePtr cur, IDMEFtarget *target);
int parse_source(xmlNodePtr cur, IDMEFsource *source);
int parse_analyzer(xmlNodePtr cur, IDMEFanalyzer *analyzer);
int parse_overflowalert(xmlNodePtr cur, IDMEFoverflowalert *overflowalert);
int parse_toolalert(xmlNodePtr cur, IDMEFtoolalert *toolalert);
int parse_correlationalert(xmlNodePtr cur,
			IDMEFcorrelationalert *correlationalert);
int parse_heartbeat(xmlNodePtr cur, IDMEFheartbeat *heartbeat);
int parse_alert(xmlNodePtr cur, IDMEFalert *alert);
int parse_message(xmlNodePtr cur, IDMEFmessage *message);

void free_webservice(IDMEFwebservice *webservice);
void free_snmpservice(IDMEFSNMPservice *snmpservice);
void free_service(IDMEFservice *service);
void free_userid(IDMEFuserid *userid);
void free_user(IDMEFuser *user);
void free_time(IDMEFtime *time);
void free_address(IDMEFaddress *address);
void free_node(IDMEFnode *node);
void free_process(IDMEFprocess *process);
void free_fileaccess(IDMEFfileaccess *fileaccess);
void free_linkage(IDMEFlinkage *linkage);
void free_inode(IDMEFinode *inode);
void free_file(IDMEFfile *file);
void free_filelist(IDMEFfilelist *filelist);
void free_source(IDMEFsource *source);
void free_target(IDMEFtarget *target);
void free_analyzer(IDMEFanalyzer *analyzer);
void free_alertident(IDMEFalertident *alertident);
void free_correlationalert(IDMEFcorrelationalert *correlationalert);
void free_toolalert(IDMEFtoolalert *toolalert);
void free_overflowalert(IDMEFoverflowalert *overflowalert);
void free_additionaldata(IDMEFadditionaldata *additionaldata);
void free_classification(IDMEFclassification *classification);
void free_impact(IDMEFimpact *impact);
void free_action(IDMEFaction *action);
void free_confidence(IDMEFconfidence *confidence);
void free_assessment(IDMEFassessment *assessment);
void free_alert(IDMEFalert *alert);
void free_heartbeat(IDMEFheartbeat *heartbeat);
void free_message(IDMEFmessage *message);

#endif /* _IDMEFXML_PARSE_H */
