/*
idmefxml_types.h: header file for libidmef data types
Author: Adam Migus, NAI Labs, (amigus@nai.com)

Copyright (c) 2001 Networks Associates Technology, Inc.
All rights reserved

This library is released under the GNU GPL and BSD software licenses.
You may choose to use one or the other, BUT NOT BOTH.  The GNU GPL
license is located in the file named COPYING.  The BSD license is located
in the file named COPYING.BSD.  Please contact us if there are any
questions.

*/

#ifndef _IDMEFXML_TYPES_H
#define _IDMEFXML_TYPES_H

/* $Id: idmefxml_types.h,v 1.1 2002/02/20 23:54:28 amigus Exp $ */

#include <time.h>
#include <libxml/parser.h>
#include <sys/time.h>

/*=================================================
 *  Enumeration definitions
 *=================================================*/

typedef enum _IDMEFspoofed {
                        SPOOFED_UNKNOWN  = 0,
			SPOOFED_YES      = 1,
			SPOOFED_NO       = 2
} IDMEFspoofed;

typedef enum _IDMEFdecoy {
                        DECOY_UNKNOWN    = 0,
		        DECOY_YES        = 1,
		        DECOY_NO         = 2
} IDMEFdecoy;

typedef enum _IDMEFaddress_category {
                        ADDRESS_CATEGORY_UNKNOWN = 0,
			ATM              = 1,
			E_MAIL           = 2,
			LOTUS_NOTES      = 3,
			MAC              = 4,
			SNA              = 5,
			VM               = 6,
			IPV4_ADDR        = 7,
			IPV4_ADDR_HEX    = 8,
			IPV4_NET         = 9,
                        IPV4_NET_MASK    = 10,
			IPV6_ADDR        = 11,
			IPV6_ADDR_HEX    = 12,
			IPV6_NET         = 13,
			IPV6_NET_MASK    = 14
} IDMEFaddress_category;

typedef enum _IDMEForigin {
                        UNKNOWN_ORIGIN   = 0,
			BUGTRAQID        = 1,
			CVE              = 2,
			VENDOR_SPECIFIC  = 3
} IDMEForigin;

typedef enum _IDMEFnode_category {
                        NODE_CATEGORY_UNKNOWN = 0,
			ADS              = 1,
			AFS              = 2,
			CODA             = 3,
			DFS              = 4,
			DNS              = 5,
			KERBEROS         = 6,
			NDS              = 7,
			NIS              = 8,
			NISPLUS          = 9,
			NT               = 10,
			WFW              = 11
} IDMEFnode_category;

typedef enum _IDMEFaction_category {
			BLOCK_INSTALLED  = 0,
			NOTIFICATION_SENT = 1,
			TAKEN_OFFLINE    = 2,
			ACTION_CATEGORY_OTHER = 3
} IDMEFaction_category;

typedef enum _IDMEFuser_category {
                        USER_CATEGORY_UNKNOWN = 0,
			APPLICATION      = 1,
			OS_DEVICE        = 2
} IDMEFuser_category;

typedef enum _IDMEFadditionaldata_type {
			BOOLEAN          = 0,
			BYTE             = 1,
			CHARACTER        = 2,
			DATETIME         = 3,
			INTEGER          = 4,
                        NTPSTAMP         = 5,
                        PORTLIST         = 6,
			REAL             = 7,
			STRING           = 8,
			XML              = 9
} IDMEFadditionaldata_type;

typedef enum _IDMEFconfidence_rating {
			CONFIDENCE_RATING_LOW     = 0,
			CONFIDENCE_RATING_MEDIUM  = 1,
			CONFIDENCE_RATING_HIGH    = 2,
                        CONFIDENCE_RATING_NUMERIC = 3
} IDMEFconfidence_rating;

typedef enum _IDMEFUserId_type {
                        CURRENT_USER     = 0,
			ORIGINAL_USER    = 1,
			TARGET_USER      = 2,
			USER_PRIVS       = 3,
			CURRENT_GROUP    = 4,
			GROUP_PRIVS      = 5,
			OTHER_PRIVS      = 6
} IDMEFUserId_type;

typedef enum _IDMEFimpact_severity {
			IMPACT_SEVERITY_LOW        = 0,
			IMPACT_SEVERITY_MEDIUM     = 1,
			IMPACT_SEVERITY_HIGH       = 2
} IDMEFimpact_severity;

typedef enum _IDMEFimpact_completion {
			FAILED           = 0,
			SUCCEEDED        = 1
} IDMEFimpact_completion;

typedef enum _IDMEFimpact_type {
			ADMIN            = 0,
			DOS              = 1,
			IMPACT_TYPE_FILE = 2,
			RECON            = 3,
                        USER             = 4,
                        IMPACT_TYPE_OTHER = 5
} IDMEFimpact_type;


typedef enum _IDMEFfile_category {
			CURRENT          = 0,
			ORIGINAL         = 1
} IDMEFfile_category;

typedef enum _IDMEFlinkage_category {
			HARD_LINK        = 0,
			MOUNT_POINT      = 1,
			REPARSE_POINT    = 2,
			SHORTCUT         = 3,
			STREAM           = 4,
			SYMBOLIC_LINK    = 5
} IDMEFlinkage_category;

/*=================================================
 *  Struct definitions
 *=================================================*/

typedef struct _IDMEFaddress {
	char *ident;
	IDMEFaddress_category category;
	char *vlan_name;	
	int vlan_num;
	char *address;
	char *netmask;
} IDMEFaddress;

typedef struct _IDMEFtime {
	char *ntpstamp;
	char *string;
	struct timeval tv;
} IDMEFtime;

typedef struct _IDMEFuserid {
	char *ident;
	IDMEFUserId_type type;
	char *name;
	int number;
} IDMEFuserid;
	
typedef struct _IDMEFuser {
	char *ident;
	IDMEFuser_category category;
	int nuserids;
	IDMEFuserid **userids;
} IDMEFuser;

typedef struct _IDMEFSNMPservice {
	char *oid;
	char *community;
	char *command;
} IDMEFSNMPservice;

typedef struct _IDMEFwebservice {
	char *url;
	char *cgi;
	char *http_method;
	int narg;
	char **arg;
} IDMEFwebservice;

typedef struct _IDMEFservice {
	char *ident;
	char *name;
	int port;
	char *portlist;
	short protocol;
	IDMEFwebservice *webservice;
	IDMEFSNMPservice *snmpservice;
} IDMEFservice;

typedef struct _IDMEFprocess {
	char *ident;
	char *name;
	int pid;
	char *path;
	int narg;
	char **arg;
	int nenv;
	char **env;
} IDMEFprocess;

typedef struct _IDMEFnode {
	char *ident;
	IDMEFnode_category category;
	char *location;
	char *name;
	int naddresses;
	IDMEFaddress **addresses;
} IDMEFnode;

typedef struct _IDMEFfileaccess {
	IDMEFuserid *userid;
	int npermissions;
	char **permissions;
} IDMEFfileaccess;

typedef struct _IDMEFlinkage {
	IDMEFlinkage_category category;
	char *name;
	char *path;
	struct _IDMEFfile *file;
} IDMEFlinkage;

typedef struct _IDMEFinode {
	char *change_time;
	int number;
	int major_device;
	int minor_device;
	int c_major_device;
	int c_minor_device;
} IDMEFinode;

typedef struct _IDMEFfile {
	char *ident;
	IDMEFfile_category category;
	char *fstype;
	char *name;
	char *path;
	char *create_time;
	char *modify_time;
	char *access_time;
	int data_size;
	int disk_size;
	int nfileaccesses;
	IDMEFfileaccess **fileaccesses;
	int nlinkages;
	IDMEFlinkage **linkages;
	IDMEFinode *inode;
} IDMEFfile;

typedef struct _IDMEFfilelist {
	int nfiles;
	IDMEFfile **files;
} IDMEFfilelist;

typedef struct _IDMEFsource {
	char *ident;
	IDMEFspoofed spoofed;
	char *interface;
	IDMEFnode *node;
	IDMEFuser *user;
	IDMEFprocess *process;
	IDMEFservice *service;
} IDMEFsource;

typedef struct _IDMEFtarget {
	char *ident;
	IDMEFdecoy decoy;
	char *interface;
	IDMEFnode *node;
	IDMEFuser *user;
	IDMEFprocess *process;
	IDMEFservice *service;
	IDMEFfilelist *filelist;
} IDMEFtarget;
	
typedef struct _IDMEFclassification {
	IDMEForigin origin;
	char *name;
	char *url;
} IDMEFclassification;

typedef struct _IDMEFimpact {
	IDMEFimpact_severity severity;
	IDMEFimpact_completion completion;
	IDMEFimpact_type type;
	char *data;
} IDMEFimpact;

typedef struct _IDMEFaction {
	IDMEFaction_category category;
	char *data;
} IDMEFaction;

typedef struct _IDMEFconfidence {
	IDMEFconfidence_rating rating;
	char *data;
} IDMEFconfidence;
	
typedef struct _IDMEFassessment {
	IDMEFimpact *impact;
	int nactions;
	IDMEFaction **actions;
	IDMEFconfidence *confidence;
} IDMEFassessment;

typedef struct _IDMEFadditionaldata {
	IDMEFadditionaldata_type type;
	char *meaning;
	char *data;
} IDMEFadditionaldata;

typedef struct _IDMEFanalyzer {
	char *analyzerid;
	char *manufacturer;
	char *model;
	char *version;
	char *class;
	char *ostype;
	char *osversion;
	IDMEFnode *node;
	IDMEFprocess *process;
} IDMEFanalyzer;

typedef struct _IDMEFalertidents {
	char *data;
	char *analyzerid;
} IDMEFalertident;

typedef struct _IDMEFtoolalert {
	char *name;
	char *command;
	int nalertidents;
	IDMEFalertident **alertidents;
} IDMEFtoolalert;

typedef struct _IDMEFoverflowalert {
	char *program;
	int size;
	char *buffer;
} IDMEFoverflowalert;

typedef struct _IDMEFcorrelationalert {
	char *name;
	int nalertidents;
	IDMEFalertident **alertidents;
} IDMEFcorrelationalert;

typedef struct _IDMEFalert {
	char *ident;
	IDMEFanalyzer *analyzer;
	IDMEFtime *createtime;
	IDMEFtime *detecttime;
	IDMEFtime *analyzertime;
	int nsources;
	IDMEFsource **sources;
	int ntargets;
	IDMEFtarget **targets;
	int nclassifications;
	IDMEFclassification **classifications;
	IDMEFassessment *assessment;
	IDMEFcorrelationalert *correlationalert;
	IDMEFtoolalert *toolalert;
	IDMEFoverflowalert *overflowalert;
	int nadditionaldatas;
	IDMEFadditionaldata **additionaldatas;
} IDMEFalert;

typedef struct _IMDEFheartbeat {
	char *ident;
	IDMEFanalyzer *analyzer;
	IDMEFtime *createtime;
	IDMEFtime *analyzertime;
	int nadditionaldatas;
	IDMEFadditionaldata **additionaldatas;
} IDMEFheartbeat;

typedef struct _IDMEFmessage {
	char *version;
	int nalerts;
	IDMEFalert **alerts;
	int nheartbeats;
	IDMEFheartbeat **heartbeats;
} IDMEFmessage;

#endif /* _IDMEFXML_TYPES_H */
