/*
parsing_example1.c: prints out contents of parsed IDMEF XML file.
Author: Adam Migus, NAI Labs, (amigus@nai.com)

Copyright (c) 2001 Networks Associates Technology, Inc.

This library is released under the GNU GPL and BSD software licenses.
You may choose to use one or the other, BUT NOT BOTH.  The GNU GPL
license is located in the file named COPYING.  The BSD license is located
in the file named COPYING.BSD.  Please contact us if there are any
questions.

$Id: parsing_example1.c,v 1.4 2002/02/20 23:54:28 amigus Exp $

*/

/*
This example program attempts to print out all the elements of an
IDMEF message.  As such it doesn't work (it core dumps) on OS's
whose printf doesn't deal well with null pointers such as Solaris.
It does however work on Linux.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <libidmef/idmefxml_parse.h>

char *program = 0;

int usage(char *p) {
	fprintf(stdout, "usage: %s <xml file>\n", p);
	return 1;
}

int
main(int argc, char **argv) {
	int i = 0, j = 0, k = 0;
	IDMEFmessage *message = 0; 
	char *p = 0;

	program = ((p = strrchr(argv[0], '/')) ? ++p : argv[0]);

	if(argc != 2) return usage(program);

	message = get_idmef_message_from_file(argv[1]);	

	if(!message) {
		fprintf(stderr, "%s: failed to parse message\n", program);
		return 1;
	}

	fprintf(stdout, "message->version=\'%s\'\n", message->version);

	for(i = 0; i < message->nalerts; i++) {
		fprintf(stdout, "message->alerts[%d]->ident=\'%s\'\n", i, message->alerts[i]->ident);
	
		if(message->alerts[i]->analyzer) {
			fprintf(stdout, "message->alerts[%d]->analyzer->analyzerid=\'%s\'\n", i, message->alerts[i]->analyzer->analyzerid);
			fprintf(stdout, "message->alerts[%d]->analyzer->manufacturer=\'%s\'\n", i, message->alerts[i]->analyzer->manufacturer);
			fprintf(stdout, "message->alerts[%d]->analyzer->model=\'%s\'\n", i, message->alerts[i]->analyzer->model);
			fprintf(stdout, "message->alerts[%d]->analyzer->version=\'%s\'\n", i, message->alerts[i]->analyzer->version);
			fprintf(stdout, "message->alerts[%d]->analyzer->class=\'%s\'\n", i, message->alerts[i]->analyzer->class);
			fprintf(stdout, "message->alerts[%d]->analyzer->ostype=\'%s\'\n", i, message->alerts[i]->analyzer->ostype);
			fprintf(stdout, "message->alerts[%d]->analyzer->osversion=\'%s\'\n", i, message->alerts[i]->analyzer->osversion);
		
			if(message->alerts[i]->analyzer->node) {
				fprintf(stdout, "message->alerts[%d]->analyzer->node->ident=\'%s\'\n", i, message->alerts[i]->analyzer->node->ident);
				fprintf(stdout, "message->alerts[%d]->analyzer->node->category=\'%s\'(%d)\n", i, node_category_as_string(message->alerts[i]->analyzer->node->category), message->alerts[i]->analyzer->node->category);
				fprintf(stdout, "message->alerts[%d]->analyzer->node->location=\'%s\'\n", i, message->alerts[i]->analyzer->node->location);
				fprintf(stdout, "message->alerts[%d]->analyzer->node->name=\'%s\'\n", i, message->alerts[i]->analyzer->node->name);
				for(j = 0; j < message->alerts[i]->analyzer->node->naddresses; j++) {
					fprintf(stdout, "message->alerts[%d]->analyzer->node->addresses[%d]->ident=\'%s\'\n", i, j, message->alerts[i]->analyzer->node->addresses[j]->ident);
					fprintf(stdout, "message->alerts[%d]->analyzer->node->addresses[%d]->category=\'%s\'(%d)\n", i, j, address_category_as_string (message->alerts[i]->analyzer->node->addresses[j]->category), message->alerts[i]->analyzer->node->addresses[j]->category);
					fprintf(stdout, "message->alerts[%d]->analyzer->node->addresses[%d]->vlan_name=\'%s\'\n", i, j, message->alerts[i]->analyzer->node->addresses[j]->vlan_name);
					fprintf(stdout, "message->alerts[%d]->analyzer->node->addresses[%d]->vlan_num=\'%d\'\n", i, j, message->alerts[i]->analyzer->node->addresses[j]->vlan_num);
					fprintf(stdout, "message->alerts[%d]->analyzer->node->addresses[%d]->address=\'%s\'\n", i, j, message->alerts[i]->analyzer->node->addresses[j]->address);
					fprintf(stdout, "message->alerts[%d]->analyzer->node->addresses[%d]->netmask=\'%s\'\n", i, j, message->alerts[i]->analyzer->node->addresses[j]->netmask);
				}
			}
	
			if(message->alerts[i]->analyzer->process) {
				fprintf(stdout, "message->alerts[%d]->analyzer->process->ident=\'%s\'\n", i, message->alerts[i]->analyzer->process->ident); fprintf(stdout, "message->alerts[%d]->analyzer->process->name=\'%s\'\n", i, message->alerts[i]->analyzer->process->name);
				fprintf(stdout, "message->alerts[%d]->analyzer->process->pid=\'%d\'\n", i, message->alerts[i]->analyzer->process->pid);
				fprintf(stdout, "message->alerts[%d]->analyzer->process->path=\'%s\'\n", i, message->alerts[i]->analyzer->process->path);
				for(j = 0; j < message->alerts[i]->analyzer->process->narg; j++) {
					fprintf(stdout, "message->alerts[%d]->analyzer->process->arg[%d]=\'%s\'\n", i, j, message->alerts[i]->analyzer->process->arg[j]);
				}
				for(j = 0; j < message->alerts[i]->analyzer->process->nenv; j++) {
					fprintf(stdout, "message->alerts[%d]->analyzer->process->env[%d]=\'%s\'\n", i, j, message->alerts[i]->analyzer->process->env[j]);
				}
			}
		}
	
		if(message->alerts[i]->createtime) {
			fprintf(stdout, "message->alerts[%d]->createtime->ntpstamp=\'%s\'\n", i, message->alerts[i]->createtime->ntpstamp);
			fprintf(stdout, "message->alerts[%d]->createtime->string=\'%s\'\n", i, message->alerts[i]->createtime->string);
			fprintf(stdout, "message->alerts[%d]->createtime->tv=\'%ld, %ld\'\n", i, message->alerts[i]->createtime->tv.tv_sec, message->alerts[i]->createtime->tv.tv_usec);
		}
	
		if(message->alerts[i]->detecttime) {
			fprintf(stdout, "message->alerts[%d]->detecttime->ntpstamp=\'%s\'\n", i, message->alerts[i]->detecttime->ntpstamp);
			fprintf(stdout, "message->alerts[%d]->detecttime->string=\'%s\'\n", i, message->alerts[i]->detecttime->string);
			fprintf(stdout, "message->alerts[%d]->detecttime->tv=\'%ld, %ld\'\n", i, message->alerts[i]->detecttime->tv.tv_sec, message->alerts[i]->detecttime->tv.tv_usec);
		}

		if(message->alerts[i]->analyzertime) {
			fprintf(stdout, "message->alerts[%d]->analyzertime->ntpstamp=\'%s\'\n", i, message->alerts[i]->analyzertime->ntpstamp);
			fprintf(stdout, "message->alerts[%d]->analyzertime->string=\'%s\'\n", i, message->alerts[i]->analyzertime->string);
			fprintf(stdout, "message->alerts[%d]->analyzertime->tv=\'%ld, %ld\'\n", i, message->alerts[i]->analyzertime->tv.tv_sec, message->alerts[i]->analyzertime->tv.tv_usec);
		}

		for(j = 0; j < message->alerts[i]->nsources; j++) {
			fprintf(stdout, "message->alerts[%d]->sources[%d]->ident=\'%s\'\n", i, j, message->alerts[i]->sources[j]->ident);
			fprintf(stdout, "message->alerts[%d]->sources[%d]->spoofed=\'%s\'(%d)\n", i, j, spoofed_as_string(message->alerts[i]->sources[j]->spoofed),
			message->alerts[i]->sources[j]->spoofed); fprintf(stdout, "message->alerts[%d]->sources[%d]->interface=\'%s\'\n", i, j, message->alerts[i]->sources[j]->interface);

			if(message->alerts[i]->sources[j]->node) {
				fprintf(stdout, "message->alerts[%d]->sources[%d]->node->ident=\'%s\'\n", i, j, message->alerts[i]->sources[j]->node->ident);
				fprintf(stdout, "message->alerts[%d]->sources[%d]->node->category=\'%s\'(%d)\n", i, j, node_category_as_string (message->alerts[i]->sources[j]->node->category), message->alerts[i]->sources[j]->node->category);
				fprintf(stdout, "message->alerts[%d]->sources[%d]->node->location=\'%s\'\n", i, j, message->alerts[i]->sources[j]->node->location);
				fprintf(stdout, "message->alerts[%d]->sources[%d]->node->name=\'%s\'\n", i, j, message->alerts[i]->sources[j]->node->name);

				for(k = 0; k < message->alerts[i]->sources[j]->node->naddresses; k++) {
					fprintf(stdout, "message->alerts[%d]->sources[%d]->node->addresses[%d]->ident=\'%s\'\n", i, j, k, message->alerts[i]->sources[j]->node->addresses[k]->ident);
					fprintf(stdout, "message->alerts[%d]->sources[%d]->node->addresses[%d]->category=\'%s\'(%d)\n", i, j, k, address_category_as_string (message->alerts[i]->sources[j]->node->addresses[k]->category), message->alerts[i]->sources[j]->node->addresses[k]->category);
					fprintf(stdout, "message->alerts[%d]->sources[%d]->node->addresses[%d]->vlan_name=\'%s\'\n", i, j, k, message->alerts[i]->sources[j]->node->addresses[k]->vlan_name);
					fprintf(stdout, "message->alerts[%d]->sources[%d]->node->addresses[%d]->vlan_num=\'%d\'\n", i, j, k, message->alerts[i]->sources[j]->node->addresses[k]->vlan_num);
					fprintf(stdout, "message->alerts[%d]->sources[%d]->node->addresses[%d]->address=\'%s\'\n", i, j, k, message->alerts[i]->sources[j]->node->addresses[k]->address);
					fprintf(stdout, "message->alerts[%d]->sources[%d]->node->addresses[%d]->netmask=\'%s\'\n", i, j, k, message->alerts[i]->sources[j]->node->addresses[k]->netmask);
				}
			}

			if(message->alerts[i]->sources[j]->user) {
				fprintf(stdout, "message->alerts[%d]->sources[%d]->user->ident=\'%s\'\n", i, j, message->alerts[i]->sources[j]->user->ident);
				fprintf(stdout, "message->alerts[%d]->sources[%d]->user->category=\'%s\'(%d)\n", i, j, user_category_as_string (message->alerts[i]->sources[j]->user->category), message->alerts[i]->sources[j]->user->category);
				for(k = 0; k < message->alerts[i]->sources[j]->user->nuserids; k++) {
					fprintf(stdout, "message->alerts[%d]->sources[%d]->user->userids[%d]->ident=\'%s\'\n", i, j, k, message->alerts[i]->sources[j]->user->userids[k]->ident);
					fprintf(stdout, "message->alerts[%d]->sources[%d]->user->userids[%d]->type=\'%s\'(%d)\n", i, j, k, userid_type_as_string (message->alerts[i]->sources[j]->user->userids[k]->type), message->alerts[i]->sources[j]->user->userids[k]->type);
					fprintf(stdout, "message->alerts[%d]->sources[%d]->user->userids[%d]->name=\'%s\'\n", i, j, k, message->alerts[i]->sources[j]->user->userids[k]->name);
					fprintf(stdout, "message->alerts[%d]->sources[%d]->user->userids[%d]->number=\'%d\'\n", i, j, k, message->alerts[i]->sources[j]->user->userids[k]->number);
				}
			}

			if(message->alerts[i]->sources[j]->process) {
				fprintf(stdout, "message->alerts[%d]->sources[%d]->process->ident=\'%s\'\n", i, j, message->alerts[i]->sources[j]->process->ident);
				fprintf(stdout, "message->alerts[%d]->sources[%d]->process->name=\'%s\'\n", i, j, message->alerts[i]->sources[j]->process->name);
				fprintf(stdout, "message->alerts[%d]->sources[%d]->process->pid=\'%d\'\n", i, j, message->alerts[i]->sources[j]->process->pid);
				fprintf(stdout, "message->alerts[%d]->sources[%d]->process->path=\'%s\'\n", i, j, message->alerts[i]->sources[j]->process->path);
				for(k = 0; k < message->alerts[i]->sources[j]->process->narg; k++) {
					fprintf(stdout, "message->alerts[%d]->sources[%d]->process->arg[%d]=\'%s\'\n", i, j, k, message->alerts[i]->sources[j]->process->arg[k]);
				}
				for(k = 0; k < message->alerts[i]->sources[j]->process->nenv; k++) {
					fprintf(stdout, "message->alerts[%d]->sources[%d]->process->env[%d]=\'%s\'\n", i, j, k, message->alerts[i]->sources[j]->process->env[k]);
				}
			}

			if(message->alerts[i]->sources[j]->service) {
				fprintf(stdout, "message->alerts[%d]->sources[%d]->service->ident=\'%s\'\n", i, j, message->alerts[i]->sources[j]->service->ident);
				fprintf(stdout, "message->alerts[%d]->sources[%d]->service->name=\'%s\'\n", i, j, message->alerts[i]->sources[j]->service->name);
				fprintf(stdout, "message->alerts[%d]->sources[%d]->service->port=\'%d\'\n", i, j, message->alerts[i]->sources[j]->service->port);
				fprintf(stdout, "message->alerts[%d]->sources[%d]->service->portlist=\'%s\'\n", i, j, message->alerts[i]->sources[j]->service->portlist);
				fprintf(stdout, "message->alerts[%d]->sources[%d]->service->protocol=\'%d\'\n", i, j, message->alerts[i]->sources[j]->service->protocol);

				if(message->alerts[i]->sources[j]->service->webservice) {
					fprintf(stdout, "message->alerts[%d]->sources[%d]->service->webservice->url=\'%s\'\n", i, j, message->alerts[i]->sources[j]->service->webservice->url);
					fprintf(stdout, "message->alerts[%d]->sources[%d]->service->webservice->cgi=\'%s\'\n", i, j, message->alerts[i]->sources[j]->service->webservice->cgi);
					fprintf(stdout, "message->alerts[%d]->sources[%d]->service->webservice->http_method=\'%s\'\n", i, j, message->alerts[i]->sources[j]->service->webservice->http_method);
					for(k = 0; k < message->alerts[i]->sources[j]->service->webservice->narg; k++) {
						fprintf(stdout, "message->alerts[%d]->sources[%d]->service->webservice->arg[%d]=\'%s\'\n", i, j, k, message->alerts[i]->sources[j]->service->webservice->arg[k]);
					}
				}
				if(message->alerts[i]->sources[j]->service->snmpservice) {
					fprintf(stdout, "message->alerts[%d]->sources[%d]->service->snmpservice->oid=\'%s\'\n", i, j, message->alerts[i]->sources[j]->service->snmpservice->oid);
					fprintf(stdout, "message->alerts[%d]->sources[%d]->service->snmpservice->community=\'%s\'\n", i, j, message->alerts[i]->sources[j]->service->snmpservice->community);
					fprintf(stdout, "message->alerts[%d]->sources[%d]->service->snmpservice->command=\'%s\'\n", i, j, message->alerts[i]->sources[j]->service->snmpservice->command);
				}
			}
		}
	
		for(j = 0; j < message->alerts[i]->ntargets; j++) {
			fprintf(stdout, "message->alerts[%d]->targets[%d]->ident=\'%s\'\n", i, j, message->alerts[i]->targets[j]->ident);
			fprintf(stdout, "message->alerts[%d]->targets[%d]->decoy=\'%s\'(%d)\n", i, j, spoofed_as_string(message->alerts[i]->targets[j]->decoy), message->alerts[i]->targets[j]->decoy);
			fprintf(stdout, "message->alerts[%d]->targets[%d]->interface=\'%s\'\n", i, j, message->alerts[i]->targets[j]->interface);

			if(message->alerts[i]->targets[j]->node) {
				fprintf(stdout, "message->alerts[%d]->targets[%d]->node->ident=\'%s\'\n", i, j, message->alerts[i]->targets[j]->node->ident);
				fprintf(stdout, "message->alerts[%d]->targets[%d]->node->category=\'%s\'(%d)\n", i, j, node_category_as_string (message->alerts[i]->targets[j]->node->category), message->alerts[i]->targets[j]->node->category);
				fprintf(stdout, "message->alerts[%d]->targets[%d]->node->location=\'%s\'\n", i, j, message->alerts[i]->targets[j]->node->location);
				fprintf(stdout, "message->alerts[%d]->targets[%d]->node->name=\'%s\'\n", i, j, message->alerts[i]->targets[j]->node->name);

				for(k = 0; k < message->alerts[i]->targets[j]->node->naddresses; k++) {
					fprintf(stdout, "message->alerts[%d]->targets[%d]->node->addresses[%d]->ident=\'%s\'\n", i, j, k, message->alerts[i]->targets[j]->node->addresses[k]->ident);
					fprintf(stdout, "message->alerts[%d]->targets[%d]->node->addresses[%d]->category=\'%s\'(%d)\n", i, j, k, address_category_as_string (message->alerts[i]->targets[j]->node->addresses[k]->category), message->alerts[i]->targets[j]->node->addresses[k]->category);
					fprintf(stdout, "message->alerts[%d]->targets[%d]->node->addresses[%d]->vlan_name=\'%s\'\n", i, j, k, message->alerts[i]->targets[j]->node->addresses[k]->vlan_name);
					fprintf(stdout, "message->alerts[%d]->targets[%d]->node->addresses[%d]->vlan_num=\'%d\'\n", i, j, k, message->alerts[i]->targets[j]->node->addresses[k]->vlan_num);
					fprintf(stdout, "message->alerts[%d]->targets[%d]->node->addresses[%d]->address=\'%s\'\n", i, j, k, message->alerts[i]->targets[j]->node->addresses[k]->address);
					fprintf(stdout, "message->alerts[%d]->targets[%d]->node->addresses[%d]->netmask=\'%s\'\n", i, j, k, message->alerts[i]->targets[j]->node->addresses[k]->netmask);
				}
			}

			if(message->alerts[i]->targets[j]->user) {
				fprintf(stdout, "message->alerts[%d]->targets[%d]->user->ident=\'%s\'\n", i, j, message->alerts[i]->targets[j]->user->ident);
				fprintf(stdout, "message->alerts[%d]->targets[%d]->user->category=\'%s\'(%d)\n", i, j, user_category_as_string (message->alerts[i]->targets[j]->user->category), message->alerts[i]->targets[j]->user->category);
				for(k = 0; k < message->alerts[i]->targets[j]->user->nuserids; k++) {
					fprintf(stdout, "message->alerts[%d]->targets[%d]->user->userids[%d]->ident=\'%s\'\n", i, j, k, message->alerts[i]->targets[j]->user->userids[k]->ident);
					fprintf(stdout, "message->alerts[%d]->targets[%d]->user->userids[%d]->type=\'%s\'(%d)\n", i, j, k, userid_type_as_string (message->alerts[i]->targets[j]->user->userids[k]->type), message->alerts[i]->targets[j]->user->userids[k]->type);
					fprintf(stdout, "message->alerts[%d]->targets[%d]->user->userids[%d]->name=\'%s\'\n", i, j, k, message->alerts[i]->targets[j]->user->userids[k]->name);
					fprintf(stdout, "message->alerts[%d]->targets[%d]->user->userids[%d]->number=\'%d\'\n", i, j, k, message->alerts[i]->targets[j]->user->userids[k]->number);
				}
			}

			if(message->alerts[i]->targets[j]->process) {
				fprintf(stdout, "message->alerts[%d]->targets[%d]->process->ident=\'%s\'\n", i, j, message->alerts[i]->targets[j]->process->ident);
				fprintf(stdout, "message->alerts[%d]->targets[%d]->process->name=\'%s\'\n", i, j, message->alerts[i]->targets[j]->process->name);
				fprintf(stdout, "message->alerts[%d]->targets[%d]->process->pid=\'%d\'\n", i, j, message->alerts[i]->targets[j]->process->pid);
				fprintf(stdout, "message->alerts[%d]->targets[%d]->process->path=\'%s\'\n", i, j, message->alerts[i]->targets[j]->process->path);
				for(k = 0; k < message->alerts[i]->targets[j]->process->narg; k++) {
					fprintf(stdout, "message->alerts[%d]->targets[%d]->process->arg[%d]=\'%s\'\n", i, j, k, message->alerts[i]->targets[j]->process->arg[k]);
				}
				for(k = 0; k < message->alerts[i]->targets[j]->process->nenv; k++) {
					fprintf(stdout, "message->alerts[%d]->targets[%d]->process->env[%d]=\'%s\'\n", i, j, k, message->alerts[i]->targets[j]->process->env[k]);
				}
			}

			if(message->alerts[i]->targets[j]->service) {
				fprintf(stdout, "message->alerts[%d]->targets[%d]->service->ident=\'%s\'\n", i, j, message->alerts[i]->targets[j]->service->ident);
				fprintf(stdout, "message->alerts[%d]->targets[%d]->service->name=\'%s\'\n", i, j, message->alerts[i]->targets[j]->service->name);
				fprintf(stdout, "message->alerts[%d]->targets[%d]->service->port=\'%d\'\n", i, j, message->alerts[i]->targets[j]->service->port);
				fprintf(stdout, "message->alerts[%d]->targets[%d]->service->portlist=\'%s\'\n", i, j, message->alerts[i]->targets[j]->service->portlist);
				fprintf(stdout, "message->alerts[%d]->targets[%d]->service->protocol=\'%d\'\n", i, j, message->alerts[i]->targets[j]->service->protocol);

				if(message->alerts[i]->targets[j]->service->webservice) {
					fprintf(stdout, "message->alerts[%d]->targets[%d]->service->webservice->url=\'%s\'\n", i, j, message->alerts[i]->targets[j]->service->webservice->url);
					fprintf(stdout, "message->alerts[%d]->targets[%d]->service->webservice->cgi=\'%s\'\n", i, j, message->alerts[i]->targets[j]->service->webservice->cgi);
					fprintf(stdout, "message->alerts[%d]->targets[%d]->service->webservice->http_method=\'%s\'\n", i, j, message->alerts[i]->targets[j]->service->webservice->http_method);
					for(k = 0; k < message->alerts[i]->targets[j]->service->webservice->narg; k++) {
						fprintf(stdout, "message->alerts[%d]->targets[%d]->service->webservice->arg[%d]=\'%s\'\n", i, j, k, message->alerts[i]->targets[j]->service->webservice->arg[k]);
					}
				}
				if(message->alerts[i]->targets[j]->service->snmpservice) {
					fprintf(stdout, "message->alerts[%d]->targets[%d]->service->snmpservice->oid=\'%s\'\n", i, j, message->alerts[i]->targets[j]->service->snmpservice->oid);
					fprintf(stdout, "message->alerts[%d]->targets[%d]->service->snmpservice->community=\'%s\'\n", i, j, message->alerts[i]->targets[j]->service->snmpservice->community);
					fprintf(stdout, "message->alerts[%d]->targets[%d]->service->snmpservice->command=\'%s\'\n", i, j, message->alerts[i]->targets[j]->service->snmpservice->command);
				}
			}
		}
	
		for(j = 0; j < message->alerts[i]->nclassifications; j++) {
			fprintf(stdout, "message->alerts[%d]->classifications[%d]->origin=\'%s\'(%d)\n", i, j, classification_origin_as_string (message->alerts[i]->classifications[j]->origin), message->alerts[i]->classifications[j]->origin);
			fprintf(stdout, "message->alerts[%d]->classifications[%d]->name=\'%s\'\n", i, j, message->alerts[i]->classifications[j]->name);
			fprintf(stdout, "message->alerts[%d]->classifications[%d]->url=\'%s\'\n", i, j, message->alerts[i]->classifications[j]->url);
		}
	
		for(j = 0; j < message->alerts[i]->nadditionaldatas; j++) {
			fprintf(stdout, "message->alerts[%d]->additionaldatas[%d]->type=\'%s\'(%d)\n", i, j, additionaldata_type_as_string (message->alerts[i]->additionaldatas[j]->type), message->alerts[i]->additionaldatas[j]->type);
			fprintf(stdout, "message->alerts[%d]->additionaldatas[%d]->meaning=\'%s\'\n", i, j, message->alerts[i]->additionaldatas[j]->meaning);
			fprintf(stdout, "message->alerts[%d]->additionaldatas[%d]->data=\'%s\'\n", i, j, message->alerts[i]->additionaldatas[j]->data);
		}
		if(message->alerts[i]->correlationalert) {	
			fprintf(stdout, "message->alerts[%d]->correlationalert->name=\'%s\'\n", i, message->alerts[i]->correlationalert->name);
		
			for(j = 0; j < message->alerts[i]->correlationalert->nalertidents; j++) {
				fprintf(stdout, "message->alerts[%d]->correlationalert->alertidents[%d]->data=\'%s\'\n", i, j, message->alerts[i]->correlationalert->alertidents[j]->data);
				fprintf(stdout, "message->alerts[%d]->correlationalert->alertidents[%d]->analyzerid=\'%s\'\n", i, j, message->alerts[i]->correlationalert->alertidents[j]->analyzerid);
			}
		}

		if(message->alerts[i]->toolalert) {	
			fprintf(stdout, "message->alerts[%d]->toolalert->name=\'%s\'\n", i, message->alerts[i]->toolalert->name);
			fprintf(stdout, "message->alerts[%d]->toolalert->command=\'%s\'\n", i, message->alerts[i]->toolalert->command);
			
			for(j = 0; j < message->alerts[i]->toolalert->nalertidents; j++) {
				fprintf(stdout, "message->alerts[%d]->toolalert->alertidents[%d]->data=\'%s\'\n", i, j, message->alerts[i]->toolalert->alertidents[j]->data);
				fprintf(stdout, "message->alerts[%d]->toolalert->alertidents[%d]->analyzerid=\'%s\'\n", i, j, message->alerts[i]->toolalert->alertidents[j]->analyzerid);
			}
		}

		if(message->alerts[i]->overflowalert) {
			fprintf(stdout, "message->alerts[%d]->overflowalert->program=\'%s\'\n", i, message->alerts[i]->overflowalert->program);
			fprintf(stdout, "message->alerts[%d]->overflowalert->size=\'%d\'\n", i, message->alerts[i]->overflowalert->size);
			fprintf(stdout, "message->alerts[%d]->overflowalert->buffer=\'%s\'\n", i, message->alerts[i]->overflowalert->buffer);
		}
	}

	for(i = 0; i < message->nheartbeats; i++) {
		fprintf(stdout, "message->heartbeats[%d]->ident=\'%s\'\n", i, message->heartbeats[i]->ident);
	
		if(message->heartbeats[i]->analyzer) {
			fprintf(stdout, "message->heartbeats[%d]->analyzer->analyzerid=\'%s\'\n", i, message->heartbeats[i]->analyzer->analyzerid);
		
			if(message->heartbeats[i]->analyzer->node) {
				fprintf(stdout, "message->heartbeats[%d]->analyzer->node->ident=\'%s\'\n", i, message->heartbeats[i]->analyzer->node->ident);
				fprintf(stdout, "message->heartbeats[%d]->analyzer->node->category=\'%s\'(%d)\n", i, node_category_as_string(message->heartbeats[i]->analyzer->node->category), message->heartbeats[i]->analyzer->node->category);
				fprintf(stdout, "message->heartbeats[%d]->analyzer->node->location=\'%s\'\n", i, message->heartbeats[i]->analyzer->node->location);
				fprintf(stdout, "message->heartbeats[%d]->analyzer->node->name=\'%s\'\n", i, message->heartbeats[i]->analyzer->node->name);
			
				for(j = 0; j < message->heartbeats[i]->analyzer->node->naddresses; j++) {
					fprintf(stdout, "message->heartbeats[%d]->analyzer->node->addresses[%d]->ident=\'%s\'\n", i, j, message->heartbeats[i]->analyzer->node->addresses[j]->ident);
					fprintf(stdout, "message->heartbeats[%d]->analyzer->node->addresses[%d]->category=\'%s\'(%d)\n", i, j, address_category_as_string (message->heartbeats[i]->analyzer->node->addresses[j]->category), message->heartbeats[i]->analyzer->node->addresses[j]->category);
					fprintf(stdout, "message->heartbeats[%d]->analyzer->node->addresses[%d]->vlan_name=\'%s\'\n", i, j, message->heartbeats[i]->analyzer->node->addresses[j]->vlan_name);
					fprintf(stdout, "message->heartbeats[%d]->analyzer->node->addresses[%d]->vlan_num=\'%d\'\n", i, j, message->heartbeats[i]->analyzer->node->addresses[j]->vlan_num);
					fprintf(stdout, "message->heartbeats[%d]->analyzer->node->addresses[%d]->address=\'%s\'\n", i, j, message->heartbeats[i]->analyzer->node->addresses[j]->address);
					fprintf(stdout, "message->heartbeats[%d]->analyzer->node->addresses[%d]->netmask=\'%s\'\n", i, j, message->heartbeats[i]->analyzer->node->addresses[j]->netmask);
				}
			}
			
			if(message->heartbeats[i]->analyzer->process) {
				fprintf(stdout, "message->heartbeats[%d]->analyzer->process->ident=\'%s\'\n", i, message->heartbeats[i]->analyzer->process->ident);
				fprintf(stdout, "message->heartbeats[%d]->analyzer->process->name=\'%s\'\n", i, message->heartbeats[i]->analyzer->process->name);
				fprintf(stdout, "message->heartbeats[%d]->analyzer->process->pid=\'%d\'\n", i, message->heartbeats[i]->analyzer->process->pid);
				fprintf(stdout, "message->heartbeats[%d]->analyzer->process->path=\'%s\'\n", i, message->heartbeats[i]->analyzer->process->path);
				for(j = 0; j < message->heartbeats[i]->analyzer->process->narg; j++) {
					fprintf(stdout, "message->heartbeats[%d]->analyzer->process->arg[%d]=\'%s\'\n", i, j, message->heartbeats[i]->analyzer->process->arg[j]);
				}
				for(j = 0; j < message->heartbeats[i]->analyzer->process->nenv; j++) {
					fprintf(stdout, "message->heartbeats[%d]->analyzer->process->env[%d]=\'%s\'\n", i, j, message->heartbeats[i]->analyzer->process->env[j]);
				}
			}
		}
	
		if(message->heartbeats[i]->createtime) {
			fprintf(stdout, "message->heartbeats[%d]->createtime->ntpstamp=\'%s\'\n", i, message->heartbeats[i]->createtime->ntpstamp);
			fprintf(stdout, "message->heartbeats[%d]->createtime->string=\'%s\'\n", i, message->heartbeats[i]->createtime->string);
			fprintf(stdout, "message->heartbeats[%d]->createtime->tv=\'%ld, %ld\'\n", i, message->heartbeats[i]->createtime->tv.tv_sec, message->heartbeats[i]->createtime->tv.tv_usec);
		}
	
		if(message->heartbeats[i]->analyzertime) {
			fprintf(stdout, "message->heartbeats[%d]->analyzertime->ntpstamp=\'%s\'\n", i, message->heartbeats[i]->analyzertime->ntpstamp);
			fprintf(stdout, "message->heartbeats[%d]->analyzertime->string=\'%s\'\n", i, message->heartbeats[i]->analyzertime->string);
			fprintf(stdout, "message->heartbeats[%d]->analyzertime->tv=\'%ld, %ld\'\n", i, message->heartbeats[i]->analyzertime->tv.tv_sec, message->heartbeats[i]->analyzertime->tv.tv_usec);
		}
	
		for(j = 0; j < message->heartbeats[i]->nadditionaldatas; j++) {
			fprintf(stdout, "message->heartbeats[%d]->additionaldatas[%d]->type=\'%s\'(%d)\n", i, j, additionaldata_type_as_string (message->heartbeats[i]->additionaldatas[j]->type), message->heartbeats[i]->additionaldatas[j]->type);
			fprintf(stdout, "message->heartbeats[%d]->additionaldatas[%d]->meaning=\'%s\'\n", i, j, message->heartbeats[i]->additionaldatas[j]->meaning);
			fprintf(stdout, "message->heartbeats[%d]->additionaldatas[%d]->data=\'%s\'\n", i, j, message->heartbeats[i]->additionaldatas[j]->data);
		}
	}

	free_message(message);

	exit(0);
}
